package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.core.BlockPos;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.MenuProvider;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.AABB;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.capability.android.IAndroidCapability;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage;
import ru.dbotthepony.mc.otm.core.Fraction;
import ru.dbotthepony.mc.otm.menu.AndroidStationMenu;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockEntityAndroidStation extends BlockEntityMatteryPowered implements MenuProvider {
	@Nullable
	@Override
	public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply) {
		return new AndroidStationMenu(containerID, inventory, this);
	}

	private final TranslatableComponent MACHINE_NAME = new TranslatableComponent("block.overdrive_that_matters.android_station");

	@Override
	protected Component getDefaultDisplayName() {
		return MACHINE_NAME;
	}

	public BlockEntityAndroidStation(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.ANDROID_STATION, p_155229_, p_155230_);
		energy = new MatteryMachineEnergyStorage(this, MatteryMachineEnergyStorage.MachineType.WORKER, new Fraction(100_000), new Fraction(250), new Fraction(250));
	}

	public static <T extends BlockEntity> void tick(Level level, BlockPos blockPos, BlockState blockState, T t) {
		if (t instanceof BlockEntityAndroidStation tile) {
			tile.batteryChargeLoop();

			if (tile.isBlockedByRedstone())
				return;

			final var pos = tile.getBlockPos();
			List<LivingEntity> entities = tile.getLevel().getEntitiesOfClass(LivingEntity.class, new AABB(pos.getX(), pos.getY(), pos.getZ(), pos.getX() + 1, pos.getY() + 2, pos.getZ() + 1));

			for (LivingEntity ent : entities) {
				final var resolver = ent.getCapability(MatteryCapability.ANDROID).resolve();

				if (resolver.isEmpty())
					continue;

				final var capability = resolver.get();

				if (!capability.isAndroid())
					continue;

				final var missing = capability.getMissingPower();

				if (missing.compareTo(Fraction.ZERO) > 0) {
					final var extract = tile.energy.extractEnergyInner(missing, true);

					if (extract.compareTo(Fraction.ZERO) > 0) {
						final var received = capability.receiveEnergyOuter(extract, true);

						if (received.compareTo(Fraction.ZERO) > 0) {
							tile.energy.extractEnergyInner(extract, false);
							capability.receiveEnergyOuter(extract, false);
						}
					}
				}
			}
		}
	}
}
