package ru.dbotthepony.mc.otm.client.screen.panels;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.ChatFormatting;
import net.minecraft.client.renderer.GameRenderer;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.item.ItemStack;
import ru.dbotthepony.mc.otm.menu.MatterySlot;
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen;
import ru.dbotthepony.mc.otm.client.render.RenderHelper;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

public class SlotPanel<T extends MatterySlot> extends AbstractSlotPanel {
	public final T slot;

	public SlotPanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, float x, float y, float width, float height, T slot) {
		super(screen, parent, x, y, width, height);
		this.slot = slot;
	}

	public SlotPanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, T slot) {
		this(screen, parent, slot.x, slot.y, 18, 18, slot);
	}

	public SlotPanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, float x, float y, T slot) {
		this(screen, parent, x, y, 18, 18, slot);
	}

	public SlotPanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, T slot, float x, float y) {
		this(screen, parent, x, y, 18, 18, slot);
	}

	@Override
	protected boolean mouseClickedInner(double mouse_x, double mouse_y, int button) {
		screen.onSlotClicked(slot, button);
		return true;
	}

	@Override
	protected boolean mouseDraggedInner(double mouse_x, double mouse_y, int mouse_code, double drag_x, double drag_y) {
		screen.onSlotMouseDragged(slot, mouse_code);
		return true;
	}

	@Override
	protected boolean mouseReleasedInner(double mouse_x, double mouse_y, int flag) {
		screen.onMouseReleasedSlot(slot, mouse_x, mouse_y, flag, false);
		return true;
	}

	@Nonnull
	@Override
	protected ItemStack getItemStack() {
		return slot.getItem();
	}

	@Override
	protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
		renderSlotBackground(stack, mouse_x, mouse_y, flag);

		var itemstack = slot.getItem();
		var carried = screen.menu.getCarried();
		var drag_hit = false;
		String count_override = null;

		if (!carried.isEmpty() && screen.isQuickCrafting() && screen.quickCraftSlots.contains(slot)) {
			if (screen.quickCraftSlots.size() == 1) {
				return;
			}

			if (AbstractContainerMenu.canItemQuickReplace(slot, carried, true) && screen.menu.canDragTo(slot)) {
				itemstack = carried.copy();
				drag_hit = true;

				AbstractContainerMenu.getQuickCraftSlotCount(screen.quickCraftSlots, screen.getQuickCraftingType(), itemstack, slot.getItem().isEmpty() ? 0 : slot.getItem().getCount());
				int k = Math.min(itemstack.getMaxStackSize(), slot.getMaxStackSize(itemstack));

				if (itemstack.getCount() > k) {
					count_override = ChatFormatting.YELLOW.toString() + k;
					itemstack.setCount(k);
				}
			} else {
				screen.quickCraftSlots.remove(slot);
				screen.recalculateQuickCraftRemaining();
			}
		}

		RenderSystem.setShader(GameRenderer::getPositionTexShader);

		if (drag_hit) {
			RenderHelper.setDrawColor(SLOT_HIGHLIGHT_DRAG);
			RenderHelper.drawRect(stack, 1, 1, getWidth() - 1, getHeight() - 1);
		}

		renderRegular(stack, itemstack, count_override);

		if (isHovered()) {
			screen.hoveredSlot = slot;
		}
	}

	@Override
	protected boolean innerRenderTooltips(PoseStack stack, float mouse_x, float mouse_y, float flag) {
		// no op, screen does it for us (completely)
		return false;
	}
}
