package ru.dbotthepony.mc.otm

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.core.Vec3i
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.Tag
import java.util.function.Consumer

operator fun Vec3i.plus(direction: Vec3i): Vec3i = this.offset(direction)
operator fun Vec3i.plus(direction: Direction): Vec3i = this.offset(direction.normal)
operator fun Vec3i.minus(direction: Vec3i): Vec3i = this.subtract(direction)
operator fun Vec3i.minus(direction: Direction): Vec3i = this.subtract(direction.normal)

operator fun BlockPos.plus(direction: Vec3i): BlockPos = this.offset(direction)
operator fun BlockPos.plus(direction: Direction): BlockPos = this.offset(direction.normal)
operator fun BlockPos.minus(direction: Vec3i): BlockPos = this.subtract(direction)
operator fun BlockPos.minus(direction: Direction): BlockPos = this.subtract(direction.normal)

operator fun Direction.unaryMinus(): Direction = this.opposite
operator fun Vec3i.unaryMinus(): Vec3i = Vec3i(-x, -y, -z)
operator fun BlockPos.unaryMinus(): BlockPos = BlockPos(-x, -y, -z)

operator fun CompoundTag.set(s: String, value: Tag) = put(s, value)
operator fun CompoundTag.set(s: String, value: Int) = putInt(s, value)
operator fun CompoundTag.set(s: String, value: Byte) = putByte(s, value)
operator fun CompoundTag.set(s: String, value: Short) = putShort(s, value)
operator fun CompoundTag.set(s: String, value: Long) = putLong(s, value)
operator fun CompoundTag.set(s: String, value: String) = putString(s, value)
operator fun CompoundTag.set(s: String, value: Boolean) = putBoolean(s, value)
operator fun CompoundTag.set(s: String, value: ByteArray) = putByteArray(s, value)
operator fun CompoundTag.set(s: String, value: LongArray) = putLongArray(s, value)

inline fun CompoundTag.ifHas(s: String, consumer: (Tag) -> Unit) {
	val tag = get(s)

	if (tag != null) {
		consumer(tag)
	}
}

fun CompoundTag.ifHas(s: String, type: Byte, consumer: (Tag) -> Unit) {
	val tag = get(s)

	if (tag != null && tag.id == type) {
		consumer(tag)
	}
}

fun <T : Tag> CompoundTag.ifHas(s: String, type: Class<T>, consumer: (T) -> Unit) {
	val tag = get(s)

	if (tag != null && tag::class.java == type) {
		consumer(tag as T)
	}
}
