package ru.dbotthepony.mc.otm.menu

import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatterBottler
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.matter.IMatterHandler
import ru.dbotthepony.mc.otm.menu.data.BooleanDataContainer
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget
import ru.dbotthepony.mc.otm.network.MatterBottlerSwitchPacket
import ru.dbotthepony.mc.otm.network.MatteryNetworking

class MatterBottlerMenu @JvmOverloads constructor(
	p_38852_: Int,
	inventory: Inventory,
	tile: BlockEntityMatterBottler? = null
) : PoweredMatteryMenu(
	Registry.Menus.MATTER_BOTTLER, p_38852_, inventory, tile
) {
	var workFlow = BooleanDataContainer()

	val progressWidget: ProgressGaugeWidget
	val matterWidget: LevelGaugeWidget
	val container = arrayOfNulls<MatterySlot>(6)

	init {
		val container = tile?.container ?: SimpleContainer(6)

		if (tile == null) {
			progressWidget = ProgressGaugeWidget(this)
			matterWidget = LevelGaugeWidget(this)
		} else {
			progressWidget = ProgressGaugeWidget(this) { tile.getWorkProgress() }
			matterWidget = LevelGaugeWidget(this, tile.matter)
		}

		for (i in 0 until container.containerSize) {
			this.container[i] = object : MatterySlot(container, i) {
				override fun mayPlace(p_40231_: ItemStack): Boolean {
					val cap = p_40231_.getCapability(MatteryCapability.MATTER).resolve()
					if (!cap.isPresent) return false

					if (workFlow.value) {
						return index < 3 && cap.get().direction != IMatterHandler.MatterDirection.EXTRACT
					} else {
						return index >= 3 && cap.get().direction != IMatterHandler.MatterDirection.RECEIVE
					}
				}
			}

			addSlot(this.container[i]!!)
		}

		addDataSlots(workFlow)
		addBatterySlot()
		addInventorySlots()
	}

	override fun broadcastChanges() {
		super.broadcastChanges()
		workFlow.value = (tile as BlockEntityMatterBottler).workFlow
	}

	override fun getWorkingSlotStart(): Int {
		return 0
	}

	override fun getWorkingSlotEnd(): Int {
		return 7
	}

	fun switchBottlerMode() {
		if (tile == null) {
			MatteryNetworking.CHANNEL.sendToServer(MatterBottlerSwitchPacket())
		} else {
			(tile as BlockEntityMatterBottler).switchWorkFlow()
		}
	}
}