package ru.dbotthepony.mc.otm.menu

import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraftforge.network.PacketDistributor
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatterPanel
import ru.dbotthepony.mc.otm.capability.matter.MatterTask
import ru.dbotthepony.mc.otm.capability.matter.PatternState
import ru.dbotthepony.mc.otm.graph.matter.IMatterGraphListener
import ru.dbotthepony.mc.otm.graph.matter.MatterNetworkGraph
import ru.dbotthepony.mc.otm.network.CancelMatterTaskPacket
import ru.dbotthepony.mc.otm.network.MatterTaskPacket
import ru.dbotthepony.mc.otm.network.MatteryNetworking
import ru.dbotthepony.mc.otm.network.PatternGridPacket
import java.util.*
import java.util.function.Consumer

class MatterPanelMenu @JvmOverloads constructor(
	p_38852_: Int,
	inventory: Inventory,
	tile: BlockEntityMatterPanel? = null
) : MatteryMenu(Registry.Menus.MATTER_PANEL, p_38852_, inventory, tile), IMatterGraphListener {
	fun taskUpdated(task: MatterTask) {
		sendNetwork(MatterTaskPacket(true, listOf(task)))
	}

	fun taskRemoved(task: MatterTask) {
		sendNetwork(MatterTaskPacket(false, listOf(task)))
	}

	// client code
	@JvmField
	var patterns = ArrayList<PatternState>()

	@JvmField
	var tasks = ArrayList<MatterTask>()

	var changeset = 0

	fun networkPatternsUpdated(patterns: Collection<PatternState>) {
		changeset++

		for (pattern in patterns) {
			val index = this.patterns.indexOf(pattern)

			if (index != -1) {
				this.patterns[index] = pattern
			} else {
				this.patterns.add(pattern)
			}
		}
	}

	fun networkPatternsRemoved(patterns: Collection<PatternState>) {
		changeset++

		for (pattern in patterns) {
			this.patterns.remove(pattern)
		}
	}

	fun networkTasksUpdated(tasks: Collection<MatterTask>) {
		changeset++

		for (task in tasks) {
			val index = this.tasks.indexOf(task)

			if (index != -1) {
				this.tasks[index] = task
			} else {
				this.tasks.add(task)
			}

			watcher_update?.accept(task)
		}
	}

	fun networkTasksRemoved(tasks: Collection<MatterTask>) {
		changeset++

		for (task in tasks) {
			this.tasks.remove(task)
			watcher_delete?.accept(task)
		}
	}

	private var watcher_delete: Consumer<MatterTask>? = null
	private var watcher_update: Consumer<MatterTask>? = null

	fun networkTaskWatcher(watcher_update: Consumer<MatterTask>, watcher_delete: Consumer<MatterTask>) {
		this.watcher_delete = watcher_delete
		this.watcher_update = watcher_update
	}

	// server code
	fun requestReplication(ply: ServerPlayer, state: PatternState, how_much: Int) {
		val tile = tile as BlockEntityMatterPanel? ?: return

		val graph = tile.getAsMatterNode().graph as MatterNetworkGraph? ?: return

		if (graph.getPattern(state) == null) {
			OverdriveThatMatters.LOGGER.error("Received replication request from {} of {}, but it is no longer in grid", ply, state)
			return
		}

		tile.addTask(state, how_much)
	}

	fun receiveTaskCancel(ply: ServerPlayer, id: UUID) {
		(tile as BlockEntityMatterPanel?)?.removeTask(id)
	}

	fun requestTaskCancel(id: UUID) {
		MatteryNetworking.CHANNEL.sendToServer(CancelMatterTaskPacket(id))
	}

	override fun onPatternAdded(state: PatternState) {
		sendNetwork(PatternGridPacket(true, listOf(state)))
	}

	override fun onPatternRemoved(state: PatternState) {
		sendNetwork(PatternGridPacket(false, listOf(state)))
	}

	override fun onPatternUpdated(new_state: PatternState, old_state: PatternState) {
		sendNetwork(PatternGridPacket(true, listOf(new_state)))
	}

	private var initial_send = false

	private var listeningGrid: MatterNetworkGraph? = null

	init {
		if (tile != null) {
			listeningGrid = tile.getAsMatterNode().graph as MatterNetworkGraph?
			tile.attachMenu(this)
			listeningGrid?.addListener(this)
		}
	}

	override fun removed(p_38940_: Player) {
		super.removed(p_38940_)
		(tile as BlockEntityMatterPanel?)?.deatachMenu(this)
		listeningGrid?.removeListener(this)
	}

	override fun broadcastChanges() {
		super.broadcastChanges()
		if (!initial_send) fullPatternBroadcast()
	}

	private fun sendNetwork(packet: Any) {
		MatteryNetworking.CHANNEL.send(PacketDistributor.PLAYER.with { inventory.player as ServerPlayer }, packet)
	}

	fun fullPatternBroadcast() {
		if (inventory.player.level.isClientSide) {
			initial_send = true
			return
		}

		val tile = tile as BlockEntityMatterPanel?

		if (tile != null) {
			val grid = tile.getAsMatterNode().graph as MatterNetworkGraph?

			if (grid != null) {
				initial_send = true
				sendNetwork(PatternGridPacket(true, grid.getStoredPatterns()))
			}

			sendNetwork(MatterTaskPacket(true, tile.allTasks))
		}
	}

	override fun getWorkingSlotStart(): Int {
		return 0
	}

	override fun getWorkingSlotEnd(): Int {
		return 0
	}
}