package ru.dbotthepony.mc.otm.menu

import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.*
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.entity.BlockEntity
import ru.dbotthepony.mc.otm.menu.widget.AbstractWidget
import java.util.function.Consumer

@JvmRecord
data class MoveResult(val merge_occured: Boolean, val remaining: ItemStack, val changed_slots: Set<Slot>)

abstract class MatteryMenu protected @JvmOverloads constructor(
	p_38851_: MenuType<*>?,
	p_38852_: Int,
	@JvmField val inventory: Inventory,
	tile: BlockEntity? = null
) : AbstractContainerMenu(p_38851_, p_38852_) {
	@JvmField
	val tile: BlockEntity?

	@JvmField
	val ply: Player = inventory.player

	@JvmField
	val mattery_widgets = ArrayList<AbstractWidget>()

	@JvmField
	val inventory_slots = ArrayList<MatterySlot>()

	@JvmField
	val main_slots = ArrayList<MatterySlot>()

	@JvmField
	protected val locked_inventory_slots: MutableSet<Int> = HashSet()
	protected fun isInventorySlotLocked(index: Int): Boolean = locked_inventory_slots.contains(index)

	@JvmField
	protected var _synchronizer: ContainerSynchronizer? = null

	override fun setSynchronizer(p_150417_: ContainerSynchronizer) {
		_synchronizer = p_150417_
		super.setSynchronizer(p_150417_)
	}

	fun addWidget(widget: AbstractWidget, consumer: Consumer<Consumer<ContainerData>>) {
		if (!mattery_widgets.contains(widget)) {
			mattery_widgets.add(widget)
			consumer.accept(Consumer { p_38885_: ContainerData -> addDataSlots(p_38885_) })
		}
	}

	fun addMainSlot(slot: MatterySlot): MatterySlot {
		addSlot(slot)
		main_slots.add(slot)
		return slot
	}

	@JvmField
	protected var inventory_slot_index_start = 0

	@JvmField
	protected var inventory_slot_index_end = 0

	init {
		this.tile = tile
	}

	@JvmOverloads
	protected fun addInventorySlots(offset: Int = 97) {
		var first = true

		for (i in 0..2) {
			for (j in 0..8) {
				val slot: MatterySlot = object : MatterySlot(inventory, j + i * 9 + 9, 8 + j * 18, 14 + i * 18) {
					override fun mayPlace(p_40231_: ItemStack): Boolean {
						return !isInventorySlotLocked(index)
					}

					override fun mayPickup(p_40228_: Player): Boolean {
						return !isInventorySlotLocked(index)
					}
				}

				inventory_slots.add(slot)
				addSlot(slot)

				if (first) {
					first = false
					inventory_slot_index_start = slot.index
				}
			}
		}

		var last: MatterySlot? = null

		for (k in 0..8) {
			last = object : MatterySlot(inventory, k, 8 + k * 18, 14 + 58) {
				override fun mayPlace(p_40231_: ItemStack): Boolean {
					return !isInventorySlotLocked(index)
				}

				override fun mayPickup(p_40228_: Player): Boolean {
					return !isInventorySlotLocked(index)
				}
			}

			addSlot(last)
			inventory_slots.add(last)
		}

		inventory_slot_index_end = last!!.index
	}

	override fun broadcastChanges() {
		for (widget in mattery_widgets) {
			widget.updateServer()
		}

		super.broadcastChanges()
	}

	override fun broadcastFullState() {
		for (widget in mattery_widgets) {
			widget.updateServer()
		}

		super.broadcastFullState()
	}

	override fun stillValid(player: Player): Boolean {
		if (tile == null) return true

		if (player.level.getBlockEntity(tile.blockPos) !== tile) {
			return false
		}

		val pos = tile.blockPos

		return player.distanceToSqr(pos.x.toDouble() + 0.5, pos.y.toDouble() + 0.5, pos.z.toDouble() + 0.5) <= 64.0
	}

	protected abstract fun getWorkingSlotStart(): Int
	protected abstract fun getWorkingSlotEnd(): Int

	// This method receive Player interactor and slot_index where Shift + Right click occurred
	// It shall return item stack that got moved
	override fun quickMoveStack(ply: Player, slot_index: Int): ItemStack {
		// this.moveItemStackTo(ItemStack, int start_slot_index, int end_slot_index, boolean iteration_order)
		// returns boolean, telling whenever ItemStack was modified (moved or sharnk)
		// false means nothing happened
		// Last boolean determine order of slot iteration, where:
		// if TRUE, iteration order is end_slot_index -> start_slot_index
		// if FALSE iteration order is start_slot_index -> end_slot_index
		val start = getWorkingSlotStart()
		val end = getWorkingSlotEnd()

		if (start == end) {
			return ItemStack.EMPTY
		}

		var moved = ItemStack.EMPTY
		val get_slot = slots[slot_index]

		if (get_slot.hasItem()) {
			val slot_item = get_slot.item
			moved = slot_item.copy()

			if (slot_index < inventory_slot_index_start) {
				// Moving FROM machine TO inventory
				if (!moveItemStackTo(slot_item, inventory_slot_index_start, inventory_slot_index_end + 1, false)) {
					return ItemStack.EMPTY
				}
			} else if (!moveItemStackTo(slot_item, start, end, false)) {
				// Moving FROM inventory TO machine
				return ItemStack.EMPTY
			}

			if (slot_item.isEmpty) {
				get_slot.set(ItemStack.EMPTY)
			} else {
				get_slot.setChanged()
			}
		}

		return moved
	}

	fun quickMoveToInventory(stack: ItemStack, simulate: Boolean): MoveResult {
		return if (inventory_slot_index_start == 0 && inventory_slot_index_end == 0) {
			MoveResult(false, stack, java.util.Set.of<Slot>())
		} else customMoveItemStackTo(
			stack,
			inventory_slot_index_start,
			inventory_slot_index_end + 1,
			false,
			simulate
		)
	}

	override fun moveItemStackTo(
		stack_to_move: ItemStack,
		initial_slot: Int,
		final_slot: Int,
		reverse_direction: Boolean
	): Boolean {
		val move_result = customMoveItemStackTo(stack_to_move, initial_slot, final_slot, reverse_direction, false)
		val remaining: ItemStack = move_result.remaining
		if (remaining.count == stack_to_move.count) return false
		stack_to_move.count = remaining.count
		return move_result.merge_occured
	}

	protected fun customMoveItemStackTo(
		_stack_to_move: ItemStack,
		initial_slot: Int,
		final_slot: Int,
		reverse_direction: Boolean,
		simulate: Boolean
	): MoveResult {
		var merge_occured = false
		var i = if (reverse_direction) final_slot - 1 else initial_slot
		val changed = HashSet<Slot>()
		val stack_to_move = _stack_to_move.copy()

		if (stack_to_move.isStackable) {
			while (!stack_to_move.isEmpty) {
				if (reverse_direction) {
					if (i < initial_slot) {
						break
					}
				} else if (i >= final_slot) {
					break
				}

				val slot = slots[i]
				val slot_stack = slot.item

				if (!slot_stack.isEmpty && ItemStack.isSameItemSameTags(stack_to_move, slot_stack)) {
					val j = slot_stack.count + stack_to_move.count
					val maxSize = slot.getMaxStackSize(stack_to_move)

					if (j <= maxSize) {
						stack_to_move.count = 0

						if (!simulate) {
							slot_stack.count = j
							slot.setChanged()
						}

						merge_occured = true
						changed.add(slot)
					} else if (slot_stack.count < maxSize) {
						stack_to_move.shrink(maxSize - slot_stack.count)

						if (!simulate) {
							slot_stack.count = maxSize
							slot.setChanged()
						}

						merge_occured = true
						changed.add(slot)
					}
				}

				i += if (reverse_direction) -1 else 1
			}
		}

		if (!stack_to_move.isEmpty) {
			i = if (reverse_direction) final_slot - 1 else initial_slot

			while (true) {
				if (reverse_direction) {
					if (i < initial_slot) {
						break
					}
				} else if (i >= final_slot) {
					break
				}

				val slot = slots[i]
				val slot_stack = slot.item

				if (slot_stack.isEmpty && slot.mayPlace(stack_to_move)) {
					if (!simulate) {
						if (stack_to_move.count > slot.maxStackSize) {
							slot.set(stack_to_move.split(slot.maxStackSize))
						} else {
							slot.set(stack_to_move.split(stack_to_move.count))
						}

						slot.setChanged()
					}

					merge_occured = true
					changed.add(slot)
					break
				}

				i += if (reverse_direction) -1 else 1
			}
		}
		return MoveResult(merge_occured, stack_to_move, changed)
	}
}